open Sharing
open Term

let rec infer (env : (var * Sharing.t) list) : Term.t -> Sharing.t = function 
    | TVar x -> 
        (try List.assoc x env with Not_found -> 
          failwith (Printf.sprintf "infer: unbound variable '%s'" x))
   | TLet (x, t1, t2) -> 
        let tau1 = infer env t1 in 
        let env = (x, tau1)::env in 
        infer env t2
    | TCst VUnit -> failwith "infer TCst (unit) : not implemented yet"
    | TCst (VInt _) -> failwith "infer TCst (int) : not implemented yet"
    | TCst (VBool _) -> failwith "infer TCst (bool) : not implemented yet"
    | TEq (t1, t2) | TLt (t1, t2) -> 
       failwith "infer TEq/Leq : not implemented yet"     
    | TAdd (t1, t2) | TSub (t1, t2) | TMul (t1, t2) -> 
       failwith "infer TAdd/Sub/Mul : not implemented yet"     
    | TIf (t1, t2, t3) -> failwith "infer TIf : not implemented yet"
    | TLetTuple (x, y, t1, t2) ->  failwith "infer TLetTuple : not implemented yet"
    | TPair (t1, t2) ->  failwith "infer TPair : not implemented yet"
    | TFun (x, t) ->  failwith "infer TFun : not implemented yet"
    | TApp (t1,t2) -> failwith "infer TApp : not implemented yet"
    | TFix (f,x,t) -> 
       failwith "infer TFix : not implemented yet"
    | TRef t1 -> 
       failwith "infer TRef : not implemented yet"
    | TDeref t1 -> 
       failwith "infer TDeref : not implemented yet"
    | TAss (t1, t2) -> 
       failwith "infer TAss : not implemented yet"
    | TSeq (t1, t2) -> 
       failwith "infer TSeq : not implemented yet"
    | TCst _ -> failwith "infer: cannot type non-base value"




