open Term
open Conv
open Type


let examples = [
  "let x = 2 in x";
  "true"; 
  "let y = 2 in false";
  "2 + (3 * 4)";
  "2 = 2";
  "if 2 = 2 then 3 else 3";
  "let x, y = (1, true) in x";
  "let x, y = (1, true) in y";
  "(fun x -> x) 42";
  "fun x -> 2";
  "fun x -> 0 < x";
  "fun x -> x";
  "fun x -> fun y -> x";
  "let apply f x = f x in apply";
  "let compose f g x = f (g x) in 
   compose";
  "let compose f g x = f (g x) in 
   let id x = x in 
   compose id id";


  "let rec f x = 2 in f 3";
  "let rec fact n = 
      if n = 0 then 1 else n * (fact (n - 1))
   in fact 5";
  "let swap c = let x, y = c in (y, x) in swap (1, 2)";
  "let s x y z = x z (y z) in s";
  "let s x y z = x z (y z) in 
   let k x y = x in 
   let i = s k k in 
   i"; 
   "ref 0; ref 1";
   "let a = ref 0 in a := 1; !a";
   "let a = ref 1 in 
    let b = ref 2 in 
    let c = ref (!a + !b) in 
    b := !c + 1;
    !b + !c";
  "let a = ref 1 in 
   let b = ref 1 in 
   let rec fibo n =
     if n = 0 then ()
     else 
       let t = !b in 
       b := !a + !b;
       a := t;
       fibo (n - 1)
   in fibo 29; !a"; 
  "let or a b = if a then true else b in 
   let rec fibo n = 
      if or (n = 0) (n = 1) then 1 else (fibo (n - 1)) + (fibo (n - 2))
   in fibo 25";
  "let rec plus n m = 
     if n = 0 then m else (plus (n - 1) m) + 1
   in 
   let rec mult n m = 
     if n = 0 then 0 else plus (mult (n - 1) m) m
   in 
   let rec power n m = 
     if m = 0 then 1 else mult n (power n (m - 1))
   in 
   power 2 12";
  "let is_prime n = 
    (* ceci est un très mauvais test de primalité *)
    let rec aux1 a = 
      let rec aux2 b = 
        if b = n then false
        else if (a * b) = n then 
          true
        else 
          aux2 (b+1)
      in 
      if a = n then true else
      if aux2 a then
        false
      else
        aux1 (a+1)
    in 
    aux1 2
  in 
  let rec next_prime n = 
    if is_prime n then n else next_prime (n+1)
  in 
  next_prime 3000";
]


let _ = 
    let print_eval x = 
    let t = term_of_string x in 
      begin
       Printf.printf "# %s\n" (string_of_term t);
       try 
         let tau = Infer.infer [] t in 
         let v = Eval.eval t in
         Printf.printf "- : %s" 
            (Type.string_of_type tau);
         flush stdout;
         Printf.printf " = %s\n" 
                (string_of_value v)
        with e -> 
          Printf.printf "Error : uncaught exception '%s'.\n" (Printexc.to_string e)
      end
  in
  List.iter print_eval  examples
