type symbol = string
type var = string 

type descr =
| Link of  t
| Repr of constr

and constr =
| Var
| Op of symbol  * t list

and t = descr ref

let rec repr t = match !t with 
  | Link t' -> let r = repr t' in t := Link r; r 
  | Repr r -> t

exception Not_unifyable_wrong_arity
exception Not_unifyable_wrong_op of string * string

let rec unify t1 t2 = 
  let r1, r2 = repr t1, repr t2 in 
  match !r1, !r2 with 
  | Link _, _ | _, Link _ -> assert false
  | Repr Var, Repr Var when r1 == r2-> ()
  | Repr Var, _ -> r1 := Link r2
  | _, Repr Var -> r2 := Link r1
  | Repr Op (f1, args1), Repr Op (f2, args2) when f1 = f2 -> 
        (try
          List.iter2 unify args1 args2
        with Invalid_argument "List.iter2" -> raise
        Not_unifyable_wrong_arity)
  | Repr Op (f1, _), Repr Op (f2, _) -> raise  (Not_unifyable_wrong_op (f1, f2))

let cyclic t = 
  let rec aux visited t = 
    if List.memq t visited then true
    else match !(repr t) with 
      | Repr (Op (f, args)) -> 
          List.exists (aux (t::visited)) args
      | _ -> false
  in aux [] t
