open Sharing

let new_var () = ref (Repr Var)
let type_unit = ref (Repr (Op ("unit",[])))
let type_bool = ref (Repr (Op ("bool",[])))
let type_int = ref (Repr (Op ("int",[])))
let build_arrow t1 t2 = ref (Repr (Op ("->",[t1;t2])))
let build_product t1 t2 = ref (Repr (Op ("*",[t1;t2])))
let build_ref t = ref (Repr (Op ("ref",[t])))

let type_variables = 
  ["'a";"'b";"'c";"'d";"'e";"'f";"'g";"'h";"'i";"'j";"'k";"'l";"'m";"'n";"'o";"'p";"'q";"'r";"'s";"'t";"'u";"'v";"'w";"'x";"'y";"'z"]

let fresh_variable l = 
  try
    List.find (fun x -> List.for_all (fun (_,y) -> y <> x)  l) type_variables
  with Not_found -> failwith "fresh: no more type variables available"

let string_of_type t = 
 let defined = ref [] in 
 let parenthesis par s = if par then Printf.sprintf "(%s)" s else s in

 let rec aux par t = 
    let r = repr t in 
    try 
      List.assq r !defined 
    with Not_found -> begin
      match !r with 
       | Link _ -> assert false
       | Repr (Op ("->", [t1;t2])) ->
           let s1 = aux true t1 in
           let s2 = aux false t2 in
           parenthesis par (Printf.sprintf "%s -> %s" s1 s2)
       | Repr (Op ("*", [t1;t2])) ->
           let s1 = aux true t1 in
           let s2 = aux true t2 in
           parenthesis par (Printf.sprintf "%s * %s" s1 s2)
       | Repr (Op (f, [])) -> Printf.sprintf "%s" f
       | Repr (Op (f, [t1])) -> Printf.sprintf "%s %s" (aux true t1) f
       | Repr (Op (f, args)) -> Printf.sprintf "%s(%s)" f
           (String.concat ", " (List.map (aux false) args))
       | Repr Var -> 
           let fresh = fresh_variable !defined in 
           defined := (r, fresh)::!defined;
           fresh
    end
  in aux false t
  
  (*
  begin
     let r = repr t in 
      match !r with 
       | Link _ -> assert false
       | Repr (Op ("->", [t1;t2])) ->
           Printf.sprintf "%s -> %s" 
              (aux true t1) (aux false t2)
       | Repr (Op ("*", [t1;t2])) ->
           Printf.sprintf "%s * %s" 
              (aux true t1) (aux true t2)
       | Repr (Op (f, [])) -> Printf.sprintf "%s" f
       | Repr (Op (f, [t1])) -> Printf.sprintf "%s %s" (aux true t1) f
       | Repr (Op (f, args)) -> Printf.sprintf "%s(%s)" f
           (String.concat ", " (List.map (aux false) args))
       | Repr Var -> 
           let fresh = fresh_variable !defined in 
           defined := (r, fresh)::!defined;
           fresh
    end *)


